==> cognito-local-3.20.0.tgz: package/lib/services/userPoolService.js <==
var __create = Object.create;
var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
var __markAsModule = (target) => __defProp(target, "__esModule", { value: true });
var __export = (target, all) => {
  __markAsModule(target);
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __reExport = (target, module2, desc) => {
  if (module2 && typeof module2 === "object" || typeof module2 === "function") {
    for (let key of __getOwnPropNames(module2))
      if (!__hasOwnProp.call(target, key) && key !== "default")
        __defProp(target, key, { get: () => module2[key], enumerable: !(desc = __getOwnPropDesc(module2, key)) || desc.enumerable });
  }
  return target;
};
var __toModule = (module2) => {
  return __reExport(__markAsModule(__defProp(module2 != null ? __create(__getProtoOf(module2)) : {}, "default", module2 && module2.__esModule && "default" in module2 ? { get: () => module2.default, enumerable: true } : { value: module2, enumerable: true })), module2);
};
__export(exports, {
  UserPoolServiceFactoryImpl: () => UserPoolServiceFactoryImpl,
  UserPoolServiceImpl: () => UserPoolServiceImpl,
  attribute: () => attribute,
  attributeValue: () => attributeValue,
  attributesAppend: () => attributesAppend,
  attributesFromRecord: () => attributesFromRecord,
  attributesInclude: () => attributesInclude,
  attributesIncludeMatch: () => attributesIncludeMatch,
  attributesRemove: () => attributesRemove,
  attributesToRecord: () => attributesToRecord,
  customAttributes: () => customAttributes,
  defaultVerifiedAttributesIfModified: () => defaultVerifiedAttributesIfModified,
  hasUnverifiedContactAttributes: () => hasUnverifiedContactAttributes,
  validatePermittedAttributeChanges: () => validatePermittedAttributeChanges
});
var import_errors = __toModule(require("../errors"));
const attribute = (name, value) => ({ Name: name, Value: value });
const attributesIncludeMatch = (attributeName, attributeValue2, attributes) => !!(attributes ?? []).find((x) => x.Name === attributeName && x.Value === attributeValue2);
const attributesInclude = (attributeName, attributes) => !!(attributes ?? []).find((x) => x.Name === attributeName);
const attributeValue = (attributeName, attributes) => {
  var _a;
  return (_a = (attributes ?? []).find((x) => x.Name === attributeName)) == null ? void 0 : _a.Value;
};
const attributesToRecord = (attributes) => (attributes ?? []).reduce((acc, attr) => __spreadProps(__spreadValues({}, acc), { [attr.Name]: attr.Value }), {});
const attributesFromRecord = (attributes) => Object.entries(attributes).map(([Name, Value]) => ({ Name, Value }));
const attributesAppend = (attributes, ...toAppend) => {
  const attributeSet = attributesToRecord(attributes);
  for (const attr of toAppend) {
    if (attr.Value) {
      attributeSet[attr.Name] = attr.Value;
    } else {
      delete attributeSet[attr.Name];
    }
  }
  return attributesFromRecord(attributeSet);
};
const attributesRemove = (attributes, ...toRemove) => (attributes == null ? void 0 : attributes.filter((x) => !toRemove.includes(x.Name))) ?? [];
const customAttributes = (attributes) => (attributes ?? []).filter((attr) => attr.Name.startsWith("custom:"));
class UserPoolServiceImpl {
  get options() {
    return this._options;
  }
  constructor(clientsDataStore, clock, dataStore, config) {
    this.clientsDataStore = clientsDataStore;
    this._options = config;
    this.clock = clock;
    this.dataStore = dataStore;
  }
  async saveAppClient(ctx, appClient) {
    ctx.logger.debug("UserPoolServiceImpl.saveAppClient");
    await this.clientsDataStore.set(ctx, ["Clients", appClient.ClientId], appClient);
  }
  async deleteAppClient(ctx, appClient) {
    ctx.logger.debug({ clientId: appClient.ClientId }, "UserPoolServiceImpl.deleteAppClient");
    await this.clientsDataStore.delete(ctx, ["Clients", appClient.ClientId]);
  }
  async deleteGroup(ctx, group) {
    ctx.logger.debug({ groupName: group.GroupName }, "UserPoolServiceImpl.deleteGroup");
    await this.dataStore.delete(ctx, ["Groups", group.GroupName]);
  }
  async deleteUser(ctx, user) {
    ctx.logger.debug({ username: user.Username }, "UserPoolServiceImpl.deleteUser");
    await this.dataStore.delete(ctx, ["Users", user.Username]);
    await this.removeUserFromAllGroups(ctx, user);
  }
  async getGroupByGroupName(ctx, groupName) {
    ctx.logger.debug("UserPoolServiceImpl.getGroupByGroupName");
    const result = await this.dataStore.get(ctx, ["Groups", groupName]);
    return result ?? null;
  }
  async getUserByUsername(ctx, username) {
    var _a, _b;
    ctx.logger.debug({ username }, "UserPoolServiceImpl.getUserByUsername");
    const aliasEmailEnabled = (_a = this.options.UsernameAttributes) == null ? void 0 : _a.includes("email");
    const aliasPhoneNumberEnabled = (_b = this.options.UsernameAttributes) == null ? void 0 : _b.includes("phone_number");
    const userByUsername = await this.dataStore.get(ctx, [
      "Users",
      username
    ]);
    if (userByUsername) {
      return userByUsername;
    }
    const users = await this.dataStore.get(ctx, "Users", {});
    for (const user of Object.values(users)) {
      if (attributesIncludeMatch("sub", username, user.Attributes)) {
        return user;
      }
      if (aliasEmailEnabled && attributesIncludeMatch("email", username, user.Attributes)) {
        return user;
      }
      if (aliasPhoneNumberEnabled && attributesIncludeMatch("phone_number", username, user.Attributes)) {
        return user;
      }
    }
    return null;
  }
  async getUserByRefreshToken(ctx, refreshToken) {
    ctx.logger.debug({ refreshToken }, "UserPoolServiceImpl.getUserByRefreshToken");
    const users = await this.listUsers(ctx);
    const user = users.find((user2) => Array.isArray(user2.RefreshTokens) && user2.RefreshTokens.includes(refreshToken));
    return user ?? null;
  }
  async listUsers(ctx) {
    ctx.logger.debug("UserPoolServiceImpl.listUsers");
    const users = await this.dataStore.get(ctx, "Users", {});
    return Object.values(users);
  }
  async updateOptions(ctx, userPool) {
    ctx.logger.debug({ userPoolId: userPool.Id }, "UserPoolServiceImpl.updateOptions");
    await this.dataStore.set(ctx, "Options", userPool);
    this._options = userPool;
  }
  async saveUser(ctx, user) {
    ctx.logger.debug({ user }, "UserPoolServiceImpl.saveUser");
    await this.dataStore.set(ctx, ["Users", user.Username], user);
  }
  async listGroups(ctx) {
    ctx.logger.debug("UserPoolServiceImpl.listGroups");
    const groups = await this.dataStore.get(ctx, "Groups", {});
    return Object.values(groups);
  }
  async addUserToGroup(ctx, group, user) {
    ctx.logger.debug({ username: user.Username, groupName: group.GroupName }, "UserPoolServiceImpl.addUserToFromGroup");
    const groupMembers = new Set(group.members ?? []);
    if (!groupMembers.has(user.Username)) {
      groupMembers.add(user.Username);
      await this.saveGroup(ctx, __spreadProps(__spreadValues({}, group), {
        LastModifiedDate: this.clock.get(),
        members: Array.from(groupMembers)
      }));
    }
  }
  async removeUserFromGroup(ctx, group, user) {
    ctx.logger.debug({ username: user.Username, groupName: group.GroupName }, "UserPoolServiceImpl.removeUserFromGroup");
    const groupMembers = new Set(group.members ?? []);
    if (groupMembers.has(user.Username)) {
      groupMembers.delete(user.Username);
      await this.saveGroup(ctx, __spreadProps(__spreadValues({}, group), {
        LastModifiedDate: this.clock.get(),
        members: Array.from(groupMembers)
      }));
    }
  }
  async removeUserFromAllGroups(ctx, user) {
    ctx.logger.debug({ username: user.Username }, "UserPoolServiceImpl.removeUserFromAllGroups");
    const groups = await this.listGroups(ctx);
    await Promise.all(groups.map((group) => this.removeUserFromGroup(ctx, group, user)));
  }
  async saveGroup(ctx, group) {
    ctx.logger.debug({ group }, "UserPoolServiceImpl.saveGroup");
    await this.dataStore.set(ctx, ["Groups", group.GroupName], group);
  }
  async listUserGroupMembership(ctx, user) {
    ctx.logger.debug({ username: user.Username }, "UserPoolServiceImpl.listUserGroupMembership");
    const groups = await this.listGroups(ctx);
    return groups.filter((x) => {
      var _a;
      return (_a = x.members) == null ? void 0 : _a.includes(user.Username);
    }).map((x) => x.GroupName).sort((a, b) => a.localeCompare(b));
  }
  async storeRefreshToken(ctx, refreshToken, user) {
    ctx.logger.debug({ refreshToken, username: user.Username }, "UserPoolServiceImpl.storeRefreshToken", refreshToken);
    const refreshTokens = Array.isArray(user.RefreshTokens) ? user.RefreshTokens : [];
    refreshTokens.push(refreshToken);
    await this.saveUser(ctx, __spreadProps(__spreadValues({}, user), {
      RefreshTokens: refreshTokens
    }));
  }
}
class UserPoolServiceFactoryImpl {
  constructor(clock, dataStoreFactory) {
    this.clock = clock;
    this.dataStoreFactory = dataStoreFactory;
  }
  async create(ctx, clientsDataStore, defaultOptions) {
    const id = defaultOptions.Id;
    ctx.logger.debug({ id }, "UserPoolServiceImpl.create");
    const dataStore = await this.dataStoreFactory.create(ctx, id, {
      Users: {},
      Options: defaultOptions
    });
    const config = await dataStore.get(ctx, "Options", defaultOptions);
    return new UserPoolServiceImpl(clientsDataStore, this.clock, dataStore, config);
  }
}
const validatePermittedAttributeChanges = (requestAttributes, schemaAttributes) => {
  for (const attr of requestAttributes) {
    const attrSchema = schemaAttributes.find((x) => x.Name === attr.Name);
    if (!attrSchema) {
      throw new import_errors.InvalidParameterError(`user.${attr.Name}: Attribute does not exist in the schema.`);
    }
    if (!attrSchema.Mutable) {
      throw new import_errors.InvalidParameterError(`user.${attr.Name}: Attribute cannot be updated. (changing an immutable attribute)`);
    }
  }
  if (attributesInclude("email_verified", requestAttributes) && !attributesInclude("email", requestAttributes)) {
    throw new import_errors.InvalidParameterError("Email is required to verify/un-verify an email");
  }
  if (attributesInclude("phone_number_verified", requestAttributes) && !attributesInclude("phone_number", requestAttributes)) {
    throw new import_errors.InvalidParameterError("Phone Number is required to verify/un-verify a phone number");
  }
  return requestAttributes;
};
const defaultVerifiedAttributesIfModified = (attributes) => {
  const attributesToSet = [...attributes];
  if (attributesInclude("email", attributes) && !attributesInclude("email_verified", attributes)) {
    attributesToSet.push(attribute("email_verified", "false"));
  }
  if (attributesInclude("phone_number", attributes) && !attributesInclude("phone_number_verified", attributes)) {
    attributesToSet.push(attribute("phone_number_verified", "false"));
  }
  return attributesToSet;
};
const hasUnverifiedContactAttributes = (userAttributesToSet) => attributeValue("email_verified", userAttributesToSet) === "false" || attributeValue("phone_number_verified", userAttributesToSet) === "false";
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  UserPoolServiceFactoryImpl,
  UserPoolServiceImpl,
  attribute,
  attributeValue,
  attributesAppend,
  attributesFromRecord,
  attributesInclude,
  attributesIncludeMatch,
  attributesRemove,
  attributesToRecord,
  customAttributes,
  defaultVerifiedAttributesIfModified,
  hasUnverifiedContactAttributes,
  validatePermittedAttributeChanges
});

==> cognito-local-3.23.3.tgz: package/lib/services/userPoolService.js <==
var __create = Object.create;
var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
var __markAsModule = (target) => __defProp(target, "__esModule", { value: true });
var __export = (target, all) => {
  __markAsModule(target);
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __reExport = (target, module2, desc) => {
  if (module2 && typeof module2 === "object" || typeof module2 === "function") {
    for (let key of __getOwnPropNames(module2))
      if (!__hasOwnProp.call(target, key) && key !== "default")
        __defProp(target, key, { get: () => module2[key], enumerable: !(desc = __getOwnPropDesc(module2, key)) || desc.enumerable });
  }
  return target;
};
var __toModule = (module2) => {
  return __reExport(__markAsModule(__defProp(module2 != null ? __create(__getProtoOf(module2)) : {}, "default", module2 && module2.__esModule && "default" in module2 ? { get: () => module2.default, enumerable: true } : { value: module2, enumerable: true })), module2);
};
__export(exports, {
  UserPoolServiceFactoryImpl: () => UserPoolServiceFactoryImpl,
  UserPoolServiceImpl: () => UserPoolServiceImpl,
  attribute: () => attribute,
  attributeValue: () => attributeValue,
  attributesAppend: () => attributesAppend,
  attributesFromRecord: () => attributesFromRecord,
  attributesInclude: () => attributesInclude,
  attributesIncludeMatch: () => attributesIncludeMatch,
  attributesRemove: () => attributesRemove,
  attributesToRecord: () => attributesToRecord,
  customAttributes: () => customAttributes,
  defaultVerifiedAttributesIfModified: () => defaultVerifiedAttributesIfModified,
  hasUnverifiedContactAttributes: () => hasUnverifiedContactAttributes,
  validatePermittedAttributeChanges: () => validatePermittedAttributeChanges
});
var import_errors = __toModule(require("../errors"));
var import_filter = __toModule(require("./filter"));
const attribute = (name, value) => ({ Name: name, Value: value });
const attributesIncludeMatch = (attributeName, attributeValue2, attributes) => !!(attributes ?? []).find((x) => x.Name === attributeName && x.Value === attributeValue2);
const attributesInclude = (attributeName, attributes) => !!(attributes ?? []).find((x) => x.Name === attributeName);
const attributeValue = (attributeName, attributes) => {
  var _a;
  return (_a = (attributes ?? []).find((x) => x.Name === attributeName)) == null ? void 0 : _a.Value;
};
const attributesToRecord = (attributes) => (attributes ?? []).reduce((acc, attr) => __spreadProps(__spreadValues({}, acc), { [attr.Name]: attr.Value }), {});
const attributesFromRecord = (attributes) => Object.entries(attributes).map(([Name, Value]) => ({ Name, Value }));
const attributesAppend = (attributes, ...toAppend) => {
  const attributeSet = attributesToRecord(attributes);
  for (const attr of toAppend) {
    if (attr.Value) {
      attributeSet[attr.Name] = attr.Value;
    } else {
      delete attributeSet[attr.Name];
    }
  }
  return attributesFromRecord(attributeSet);
};
const attributesRemove = (attributes, ...toRemove) => (attributes == null ? void 0 : attributes.filter((x) => !toRemove.includes(x.Name))) ?? [];
const customAttributes = (attributes) => (attributes ?? []).filter((attr) => attr.Name.startsWith("custom:"));
class UserPoolServiceImpl {
  get options() {
    return this._options;
  }
  constructor(clientsDataStore, clock, dataStore, config) {
    this.clientsDataStore = clientsDataStore;
    this._options = config;
    this.clock = clock;
    this.dataStore = dataStore;
  }
  async saveAppClient(ctx, appClient) {
    ctx.logger.debug("UserPoolServiceImpl.saveAppClient");
    await this.clientsDataStore.set(ctx, ["Clients", appClient.ClientId], appClient);
  }
  async deleteAppClient(ctx, appClient) {
    ctx.logger.debug({ clientId: appClient.ClientId }, "UserPoolServiceImpl.deleteAppClient");
    await this.clientsDataStore.delete(ctx, ["Clients", appClient.ClientId]);
  }
  async deleteGroup(ctx, group) {
    ctx.logger.debug({ groupName: group.GroupName }, "UserPoolServiceImpl.deleteGroup");
    await this.dataStore.delete(ctx, ["Groups", group.GroupName]);
  }
  async deleteUser(ctx, user) {
    ctx.logger.debug({ username: user.Username }, "UserPoolServiceImpl.deleteUser");
    await this.dataStore.delete(ctx, ["Users", user.Username]);
    await this.removeUserFromAllGroups(ctx, user);
  }
  async getGroupByGroupName(ctx, groupName) {
    ctx.logger.debug("UserPoolServiceImpl.getGroupByGroupName");
    const result = await this.dataStore.get(ctx, ["Groups", groupName]);
    return result ?? null;
  }
  async getUserByUsername(ctx, username) {
    var _a, _b;
    ctx.logger.debug({ username }, "UserPoolServiceImpl.getUserByUsername");
    const aliasEmailEnabled = (_a = this.options.UsernameAttributes) == null ? void 0 : _a.includes("email");
    const aliasPhoneNumberEnabled = (_b = this.options.UsernameAttributes) == null ? void 0 : _b.includes("phone_number");
    const userByUsername = await this.dataStore.get(ctx, [
      "Users",
      username
    ]);
    if (userByUsername) {
      return userByUsername;
    }
    const users = await this.dataStore.get(ctx, "Users", {});
    for (const user of Object.values(users)) {
      if (attributesIncludeMatch("sub", username, user.Attributes)) {
        return user;
      }
      if (aliasEmailEnabled && attributesIncludeMatch("email", username, user.Attributes)) {
        return user;
      }
      if (aliasPhoneNumberEnabled && attributesIncludeMatch("phone_number", username, user.Attributes)) {
        return user;
      }
    }
    return null;
  }
  async getUserByRefreshToken(ctx, refreshToken) {
    ctx.logger.debug({ refreshToken }, "UserPoolServiceImpl.getUserByRefreshToken");
    const users = await this.listUsers(ctx);
    const user = users.find((user2) => Array.isArray(user2.RefreshTokens) && user2.RefreshTokens.includes(refreshToken));
    return user ?? null;
  }
  async listUsers(ctx, filter) {
    ctx.logger.debug("UserPoolServiceImpl.listUsers");
    const filterConfig = new import_filter.FilterConfig({
      username: import_filter.FilterConfig.caseSensitive((x) => x.Username),
      email: import_filter.FilterConfig.caseSensitive((x) => attributeValue("email", x.Attributes)),
      phone_number: import_filter.FilterConfig.caseSensitive((x) => attributeValue("phone_number", x.Attributes)),
      name: import_filter.FilterConfig.caseSensitive((x) => attributeValue("name", x.Attributes)),
      given_name: import_filter.FilterConfig.caseSensitive((x) => attributeValue("given_name", x.Attributes)),
      family_name: import_filter.FilterConfig.caseSensitive((x) => attributeValue("family_name", x.Attributes)),
      preferred_username: import_filter.FilterConfig.caseSensitive((x) => attributeValue("preferred_username", x.Attributes)),
      "cognito:user_status": import_filter.FilterConfig.caseInsensitive((x) => x.UserStatus),
      status: import_filter.FilterConfig.caseSensitive((x) => x.Enabled),
      sub: import_filter.FilterConfig.caseSensitive((x) => attributeValue("sub", x.Attributes))
    });
    const users = await this.dataStore.get(ctx, "Users", {});
    return Object.values(users).filter(filterConfig.parse(filter));
  }
  async updateOptions(ctx, userPool) {
    ctx.logger.debug({ userPoolId: userPool.Id }, "UserPoolServiceImpl.updateOptions");
    await this.dataStore.set(ctx, "Options", userPool);
    this._options = userPool;
  }
  async saveUser(ctx, user) {
    ctx.logger.debug({ user }, "UserPoolServiceImpl.saveUser");
    await this.dataStore.set(ctx, ["Users", user.Username], user);
  }
  async listGroups(ctx) {
    ctx.logger.debug("UserPoolServiceImpl.listGroups");
    const groups = await this.dataStore.get(ctx, "Groups", {});
    return Object.values(groups);
  }
  async addUserToGroup(ctx, group, user) {
    ctx.logger.debug({ username: user.Username, groupName: group.GroupName }, "UserPoolServiceImpl.addUserToFromGroup");
    const groupMembers = new Set(group.members ?? []);
    if (!groupMembers.has(user.Username)) {
      groupMembers.add(user.Username);
      await this.saveGroup(ctx, __spreadProps(__spreadValues({}, group), {
        LastModifiedDate: this.clock.get(),
        members: Array.from(groupMembers)
      }));
    }
  }
  async removeUserFromGroup(ctx, group, user) {
    ctx.logger.debug({ username: user.Username, groupName: group.GroupName }, "UserPoolServiceImpl.removeUserFromGroup");
    const groupMembers = new Set(group.members ?? []);
    if (groupMembers.has(user.Username)) {
      groupMembers.delete(user.Username);
      await this.saveGroup(ctx, __spreadProps(__spreadValues({}, group), {
        LastModifiedDate: this.clock.get(),
        members: Array.from(groupMembers)
      }));
    }
  }
  async removeUserFromAllGroups(ctx, user) {
    ctx.logger.debug({ username: user.Username }, "UserPoolServiceImpl.removeUserFromAllGroups");
    const groups = await this.listGroups(ctx);
    await Promise.all(groups.map((group) => this.removeUserFromGroup(ctx, group, user)));
  }
  async saveGroup(ctx, group) {
    ctx.logger.debug({ group }, "UserPoolServiceImpl.saveGroup");
    await this.dataStore.set(ctx, ["Groups", group.GroupName], group);
  }
  async listUserGroupMembership(ctx, user) {
    ctx.logger.debug({ username: user.Username }, "UserPoolServiceImpl.listUserGroupMembership");
    const groups = await this.listGroups(ctx);
    return groups.filter((x) => {
      var _a;
      return (_a = x.members) == null ? void 0 : _a.includes(user.Username);
    }).map((x) => x.GroupName).sort((a, b) => a.localeCompare(b));
  }
  async storeRefreshToken(ctx, refreshToken, user) {
    ctx.logger.debug({ refreshToken, username: user.Username }, "UserPoolServiceImpl.storeRefreshToken", refreshToken);
    const refreshTokens = Array.isArray(user.RefreshTokens) ? user.RefreshTokens : [];
    refreshTokens.push(refreshToken);
    await this.saveUser(ctx, __spreadProps(__spreadValues({}, user), {
      RefreshTokens: refreshTokens
    }));
  }
}
class UserPoolServiceFactoryImpl {
  constructor(clock, dataStoreFactory) {
    this.clock = clock;
    this.dataStoreFactory = dataStoreFactory;
  }
  async create(ctx, clientsDataStore, defaultOptions) {
    const id = defaultOptions.Id;
    ctx.logger.debug({ id }, "UserPoolServiceImpl.create");
    const dataStore = await this.dataStoreFactory.create(ctx, id, {
      Users: {},
      Options: defaultOptions
    });
    const config = await dataStore.get(ctx, "Options", defaultOptions);
    return new UserPoolServiceImpl(clientsDataStore, this.clock, dataStore, config);
  }
}
const validatePermittedAttributeChanges = (requestAttributes, schemaAttributes) => {
  for (const attr of requestAttributes) {
    const attrSchema = schemaAttributes.find((x) => x.Name === attr.Name);
    if (!attrSchema) {
      throw new import_errors.InvalidParameterError(`user.${attr.Name}: Attribute does not exist in the schema.`);
    }
    if (!attrSchema.Mutable) {
      throw new import_errors.InvalidParameterError(`user.${attr.Name}: Attribute cannot be updated. (changing an immutable attribute)`);
    }
  }
  if (attributesInclude("email_verified", requestAttributes) && !attributesInclude("email", requestAttributes)) {
    throw new import_errors.InvalidParameterError("Email is required to verify/un-verify an email");
  }
  if (attributesInclude("phone_number_verified", requestAttributes) && !attributesInclude("phone_number", requestAttributes)) {
    throw new import_errors.InvalidParameterError("Phone Number is required to verify/un-verify a phone number");
  }
  return requestAttributes;
};
const defaultVerifiedAttributesIfModified = (attributes) => {
  const attributesToSet = [...attributes];
  if (attributesInclude("email", attributes) && !attributesInclude("email_verified", attributes)) {
    attributesToSet.push(attribute("email_verified", "false"));
  }
  if (attributesInclude("phone_number", attributes) && !attributesInclude("phone_number_verified", attributes)) {
    attributesToSet.push(attribute("phone_number_verified", "false"));
  }
  return attributesToSet;
};
const hasUnverifiedContactAttributes = (userAttributesToSet) => attributeValue("email_verified", userAttributesToSet) === "false" || attributeValue("phone_number_verified", userAttributesToSet) === "false";
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  UserPoolServiceFactoryImpl,
  UserPoolServiceImpl,
  attribute,
  attributeValue,
  attributesAppend,
  attributesFromRecord,
  attributesInclude,
  attributesIncludeMatch,
  attributesRemove,
  attributesToRecord,
  customAttributes,
  defaultVerifiedAttributesIfModified,
  hasUnverifiedContactAttributes,
  validatePermittedAttributeChanges
});
